



* create the data sets;

options nocenter ps=100 ls=100 formdlim='-';

%include 'd:\epi697\williams\williams_data.sas' / source2;


* EXAMPLE 1
select three variables from data set ADMITS
;

data selvar1 ;
set admits (keep = pt_id admdate disdate);
run;

title 'SELVAR1';
proc print data=selvar1;
run;

proc sql;
create table selvar2 as
select pt_id, admdate, disdate
from admits;
quit;

title 'SELVAR2';
proc print data=selvar2;
run;


* EXTRA
no new data set - print directly from ex admits
;

title 'PRINT SELECTED VARIABLES DIRECTLY FROM ADMITS - PROC PRINT';
proc print data=admits;
var pt_id admdate disdate;
run;

title 'PRINT SELECTED VARIABLES DIRECTLY FROM ADMITS - PROC SQL';
proc sql;
select pt_id, admdate, disdate, admdate as a_day format=downame., disdate as d_day format=downame.
from admits ;
quit;



title 'PRINT ALL VARIABLES DIRECTLY FROM ADMITS - PROC PRINT';
proc print data=admits;
run;

title 'PRINT ALL VARIABLES DIRECTLY FROM ADMITS - PROC SQL';
proc sql;
select *
from admits;
quit;


proc sort data=admits out=new;
by md;
run;

title 'PROC SORT BY MD';
proc print data=new;
run;

title 'PRINT ALL VARIABLES DIRECTLY FROM ADMITS - PROC SQL SORTS OUTPUT, NOT THE DATA SET';
proc sql;
select *
from admits
order by md;
quit;







* EXAMPLE 2A
select observations from data set ADMITS
based on the value of an already existing variable (HOSP)
;

data vahosp1 ;
set admits;
where hosp eq 3;
* if hosp eq 3;
run;

title 'VAHOSP1';
proc print data=vahosp1;
run;

proc sql;
*proc sql feedback;
*proc sql _method;
*proc sql _tree;
create table vahosp2 as
select *
from admits
where hosp eq 3;
quit;

title 'VAHOSP2';
proc print data=vahosp2;
run;



* EXAMPLE 2B
create a variable called DXGRP that categorizes the primary diagnosis
into one of three categories (myocardial infarction [MI], congestive
heart failure [CHF] or other), based on the ICD-9 code
;

data mi1 ;
* why use a LENGTH statement?;
length dxgrp $5;

set admits;

if primdx eq: '410' then dxgrp = 'MI';
else
if primdx eq: '428' then dxgrp = 'CHF';
else                     dxgrp = 'OTHER';
run;

title 'MI1';
proc print data=mi1;
run;

* EXTRA
use a SELECT statement instead of IF-THEN-ELSE
;

data mi1s;
length dxgrp $5;
set admits;
select;
   when (primdx eq : '410') dxgrp = 'MI';
   when (primdx eq : '428') dxgrp = 'CHF';
   otherwise                dxgrp = 'OTHER';
end;
run;

title 'MI1S';
proc print data=mi1s;
run;

proc sql;
create table mi2 as
select *,
       case
	      when primdx like '410%' then 'MI'
	      when primdx like '428%' then 'CHF'
	      else                         'OTHER'
	   end as dxgrp
from admits
quit;

title 'MI2';
proc print data=mi2;
run;

run;

* alternative --- eq : cannot be used, but eqt can;

proc sql;
create table mi2 as
select *,
       case
	      when primdx eqt '410' then 'MI'
	      when primdx eqt '428' then 'CHF'
	      else                       'OTHER'
	   end as dxgrp
from admits
quit;

title 'MI2';
proc print data=mi2;
run;

* alternative ... only two levels;

data dtwo_choices;
set admits;
dxgrp = ifc(primdx eq : '410', 'MI','OTHER');
run;

title 'TWO CHOICES - DATA STEP';
proc print data=dtwo_choices;
run;

proc sql;
create table stwo_choices as
select *, ifc(primdx eqt '410', 'MI','OTHER') as dxgrp
from admits;
quit;

title 'TWO CHOICES - PROC SQL';
proc print data=stwo_choices;
run;








* EXAMPLE 2C
create a variable corresponding to the number of days of the hospital
stay and select only those stays with duration of at least 14 days
;

data twowks1 ;
attrib los label='length of stay';
set admits (keep = pt_id hosp admdate disdate) ;
los = (disdate - admdate) + 1;
if los ge 14;
run;

title 'TWOWKS1';
proc print data=twowks1;
run;

proc sql;
create table twowks2 as
select pt_id, hosp, admdate, disdate,
       (disdate - admdate) + 1 as los label='length of stay'
from admits
where calculated los ge 14;
quit;

title 'TWOWKS2';
proc print data=twowks2 label;
run;

* EXTRA
no name used for the new variable
;

proc sql;
create table twowks2 as
select pt_id, hosp, admdate, disdate,  (disdate - admdate) + 1
from admits
where (disdate - admdate) + 1 ge 14;
quit;

title 'TWOWKS2';
proc print data=twowks2;
run;





* EXAMPLE 2D
Returning to the example of selecting admissions to the Veterans
Administration hospital, lets say we didnt know which value of the
HOSP variable corresponded to the VA hospital.  The information that
provides a cross-walk between the hospital identifier code and the
hospital name is in the HOSPITALS data set.
;

* options mergenoby=error;


proc sort data = admits out=admits;
by hosp ;
run;

data vahosp1d (drop = hospname) ;
merge admits (in=adm)
      hospital (in=va keep = hosp_id hospname
                rename = (hosp_id=hosp)
                where = (hospname eq: 'Veteran'));
by hosp ;
* change default merge behavior - what happens without the IF statement;
if adm and va;
run;

proc sort data=vahosp1d;
by pt_id admdate;
run;

title 'VAHOSP1D';
proc print data=vahosp1d;
run;



proc sql ;
create table vahosp2d as
select * from admits
where hosp eq (select hosp_id
               from hospital
               where hospname like "Vet%")
order by pt_id, admdate ;
quit;

title 'VAHOSP2D';
proc print data=vahosp2d;
run;




* EXAMPLE 3
count admissions, min/max length of stay for each patient with at least one admission
;

data admsum1 ;
* what does the SET plus BY allow you to do? what is assumed here?;
set admits ;
by pt_id;

** (1) initialization;
if first.pt_id then do;
   nstays = 0;
   minlos = .;
   maxlos = .;
end;

** (2) accumulation;
nstays + 1;
los = (disdate - admdate) + 1;
minlos = min(of minlos, los) ;
maxlos = max(of maxlos, los) ;

** (3) output;
if last.pt_id then output ;

retain nstays minlos maxlos ;
keep pt_id minlos maxlos ;
run;

title 'ADMSUM1';
proc print data=admsum1;
run;

* EXTRA ... DOW;

data admsum1a;
do nstays=1 by 1 until(last.pt_id);
   set admits ;
   by pt_id;
   minlos = min(of minlos, (disdate - admdate) + 1) ;
   maxlos = max(of maxlos, (disdate - admdate) + 1) ;
end;
keep pt_id nstays minlos maxlos ;
run;

title 'ADMSUM1A';
proc print data=admsum1a;
run;


* EXTRA ... uses a data set view plus a procedure;

data newlos / view=newlos;
set admits;
los = (disdate - admdate) + 1;
run;

proc summary data=newlos;
var los;
class pt_id;
output out=minmax (drop=_type_ rename=(_freq_=nstays)) min=minlos max=maxlos
run;

title 'MINMAX FROM PROC SUMMARY';
proc print data=minmax;
run;

* use SQL and GROUP + COUNT(*);

proc sql;
create table admsum2 as
select pt_id, count(*) as nstays,
       min(disdate - admdate + 1) as minlos,
       max(disdate - admdate + 1) as maxlos
from admits
group by pt_id ;
quit;

title 'ADMSUM2';
proc print data=admsum2;
run;


* EXTRA
add variables to the data set --- what happens? notice the LOG mentions REMERGE;
;

proc sql;
create table admsum2a2 as
select pt_id, count(*) as nstays, (disdate - admdate + 1) as los,
       min(calculated los) as minlos,
       max(calculated los) as maxlos
from admits
group by pt_id ;
quit;

title 'ADMSUM2A2';
proc print data=admsum2a2;
run;
























* EXAMPLE 4A
select observations based on summary functions
blood pressure outliers - ge +/- two standard deviations from the data set mean
;

proc summary data= admits;
var bp_sys ;
output out=bpstats (keep=mean_sbp sd_sbp) mean=mean_sbp std=sd_sbp;
run;

title 'BPSTATS';
proc print data=bpstats;
run;

* EXTRA;

proc summary data= admits;
var bp_sys ;
output out=bpstats  mean= std= / autoname;
run;

title 'BPSTATS';
proc print data=bpstats;
run;

* back to example;

proc summary data= admits;
var bp_sys ;
output out=bpstats (keep=mean_sbp sd_sbp) mean=mean_sbp std=sd_sbp;
run;

title 'BPSTATS';
proc print data=bpstats;
run;


data hi_sys1 ;
* why is the RETAIN statement necessary?;
retain high low;
set bpstats admits ;
if _n_ eq 1 then do;
   high = mean_sbp + 2*(sd_sbp) ;
   low  = mean_sbp - 2*(sd_sbp) ;
   delete;
end;
if (bp_sys ge high) or (bp_sys le low) ;
drop mean_sbp sd_sbp high low ;
run;

title 'HI_SYS1';
proc print data=hi_sys1;
run;


* EXTRA --- DOW ... no RETAIN;

proc summary data= admits;
var bp_sys ;
output out=bpstats (keep=mean_sbp sd_sbp) mean=mean_sbp std=sd_sbp;
run;


data hi_sys11 ;
* why is the RETAIN statement NOT necessary?;
set bpstats;
high = mean_sbp + 2*(sd_sbp) ;
low  = mean_sbp - 2*(sd_sbp) ;

do until (done);
  set admits end=done;
  if (bp_sys ge high) or (bp_sys le low) then output;
end;

drop mean_sbp sd_sbp high low ;
run;

title 'HI_SYS1';
proc print data=hi_sys11;
run;



* EXTRA use some macro variables;

proc summary data= admits ;
var bp_sys ;
output out=bpstats (keep=mean_sbp sd_sbp) mean=mean_sbp std=sd_sbp;
run;

data _null_;
set bpstats;
call symputx('high',mean_sbp + 2*(sd_sbp));
call symputx('low' ,mean_sbp - 2*(sd_sbp)) ;
run;

%put &high &low;

data hi_sys1;
set admits;
if (bp_sys ge &high) or (bp_sys le &low) ;
run;

title 'HI_SYS1';
proc print data=hi_sys1;
run;


* SQL solution;

proc sql ;
create table hi_sys2 as
select * from admits
   where bp_sys ge (select (mean(bp_sys) + 2*std(bp_sys)) from admits)
         or
         bp_sys le (select (mean(bp_sys) - 2*std(bp_sys)) from admits);
quit;

title 'HI_SYS2';
proc print data=hi_sys2;
run;


* EXTRA - MACRO VARS IN PROC SQL;

proc sql;
select (mean(bp_sys) + 2*std(bp_sys)), (mean(bp_sys) - 2*std(bp_sys)) into :hi, :lo from admits;
create table hi_sys2 as
select * from admits 
where bp_sys ge &hi or bp_sys le &lo;
quit;

title 'HI_SYS2';
proc print data=hi_sys2;
run;

%put &hi &lo;




* EXAMPLE 4B
select observations based on summary functions
blood pressure outliers - ge +/- two standard deviations from the discharge destination mean
a "remerge" since statistics are calculated by discharge destination
;

proc summary data= admits nway;
class dest ;
var bp_sys ;
output out=bpstats2 mean=mean_sys std=sd_sys ;
run;

title 'BPSTATS2';
proc print data=bpstats2;
run;

proc sort data = admits out=admits;
by dest ;
run;

data hi_sys3 ;
merge admits (keep = pt_id bp_sys bp_dia dest)
      bpstats2 (keep = dest mean_sys sd_sys);
by dest ;
if bp_sys ge mean_sys + 2*(sd_sys) or
   bp_sys le mean_sys - 2*(sd_sys) ;
format mean_sys sd_sys 6.2;
run;

title 'HI_SYS3';
proc print data=hi_sys3;
run;

*
SKIP ... alternative data step method
multiple SET statements (no MERGE)
why NO RETAIN statement
;

proc sort data=admits out=admits;
by dest;
run;

data hi_sys3;
*put _all_;
set admits (keep = pt_id bp_sys bp_dia dest);
by dest;
if first.dest then set bpstats2 (keep = dest mean_sys sd_sys);
if bp_sys ge mean_sys + 2*(sd_sys) or
   bp_sys le mean_sys - 2*(sd_sys) ;
format mean_sys sd_sys 6.2;
run;

title 'HI_SYS3 - ALTERNATIVE DATA STEP METHOD';
proc print data=hi_sys3;
run;


* EXTRA ... DOW;

proc sort data=admits out=admits;
by dest;
run;

data hi_sys4;
set bpstats2 (keep = dest mean_sys sd_sys);
lo = mean_sys - 2*(sd_sys);
hi = mean_sys + 2*(sd_sys);
do until (last.dest);
   set admits (keep = pt_id bp_sys bp_dia dest);
   by dest;
   if bp_sys ge hi or bp_sys le lo then output;
end;
format mean_sys sd_sys 6.2;
run;

title 'HI_SYS4 - DOW';
proc print data=hi_sys3;
run;


* SQL solution;

proc sql;
create table hi_sys4 as
select pt_id, bp_sys, bp_dia, dest,
       mean(bp_sys) as mean_sys format=6.2,
       std(bp_sys)  as sd_sys   format=6.2
from admits
group by dest
having bp_sys ge (mean(bp_sys) + 2*std(bp_sys))
    or bp_sys le (mean(bp_sys) - 2*std(bp_sys)) ;
quit;

title 'HI_SYS4';
proc print data=hi_sys4;
run;




proc sql;
create table hi_sys4 as
select pt_id, bp_sys, bp_dia, dest,
       mean(bp_sys) as mean_sys format=6.2,
       std(bp_sys)  as sd_sys   format=6.2
from admits
group by dest
having bp_sys ge (mean_sys + 2*sd_sys)
    or bp_sys le (mean_sys - 2*sd_sys) ;
quit;

title 'HI_SYS4';
proc print data=hi_sys4;
run;




* EXAMPLE 4C
identify duplicates
;

proc print data=doctors;
run;

/*proc sort data=doctors;*/
/*by lastname;*/
/*run;*/

data selmd1 ;
set doctors (rename=(hospadm=hospital));
by md_id ;
if not (first.md_id and last.md_id) ;
/*by lastname ;*/
/*if not (first.lastname and last.lastname) ;*/
run;

proc sort data=selmd1 ;
by lastname hospital;
run;

title 'SELMD1';
proc print data=selmd1;
run;

* no sort needed;

proc sql ;
create table selmd2 as
select md_id, lastname, hospadm as hospital
from doctors
group by md_id
having count(*) ge 2
order by lastname, hospital;
quit;

title 'SELMD2';
proc print data=selmd2;
run;


* SKIP ... EXTRA
if all you want is a report
do not create a table
;

title 'SELMD2 - NO DATA SET CREATED';
proc sql ;
select md_id, lastname, hospadm as hospital
from doctors
group by md_id
having count(*) ge 2
order by lastname, hospital;
quit;


* EXTRA ... NEW IN V9.3;

proc sort data=doctors out=single_docs dupout=dup_docs nodupkey;
by md_id;
run;

title 'SINGLE MDS USING PROC SORT AND NODUPKEY';
proc print data=single_docs;
run;


title 'DUPLICATE MDS USING PROC SORT AND NOUDUPKEY';
proc print data=dup_docs;
run;



proc sort data=doctors out=dup_docs uniqueout=single_docs nouniquekey;
by md_id;
run;

title 'SINGLE MDS USING PROC SORT AND NOUNIQUEKEY';
proc print data=single_docs;
run;


title 'DUPLICATE MDS USING PROC SORT AND NOUNIQUEKEY';
proc print data=dup_docs;
run;




* EXAMPLE 5A
create two data sets from one (admits in 1997 and 1998)
;

data admits97 admits98;
set admits;
if year(admdate) = 1997 then output admits97;
else
if year(admdate) = 1998 then output admits98;
run;

title 'ADMITS97';
proc print data=admits97;
run;

title 'ADMITS98';
proc print data=admits98;
run;


* EXTRA;
data 
admits97 (where=(year(admdate) eq 1997))
admits98 (where=(year(admdate) eq 1998))
;
set admits;
run;

title 'ADMITS97';
proc print data=admits97;
run;

title 'ADMITS98';
proc print data=admits98;
run;



proc sql;
create table adm97 as select * from admits where year(admdate) = 1997;
create table adm98 as select * from admits where year(admdate) = 1998;
quit;


title 'ADM97';
proc print data=adm97;
run;

title 'ADM98';
proc print data=adm98;
run;



* EXAMPLE 5B
concatenate (actually INTERLEAVE) two data sets and create a third data set in patient id order
;

data alladm1;
set admits97 admits98;
by pt_id;
run;

title 'ALLADM1';
proc print data=alladm1;
run;

proc sql;
create table alladm2 as
   select * from adm97
      union
   select * from adm98
order by pt_id;
quit;

title 'ALLADM2';
proc print data=alladm2;
run;


* EXTRA ... HOW DOES UNION ARRANGE 'COLUMNS';

proc contents data=adm97 varnum;
ods select position;
run;

data adm97;
retain hosp;
set adm97;
run;

proc contents data=adm97 varnum;
ods select position;
run;

proc contents data=adm98 varnum;
ods select position;
run;

title 'ADM97 ... VARIABLE ORDER CHANGED';
proc print data=adm97;
run;

title 'ADM98';
proc print data=adm98;
run;

proc sql;
create table alladm2 as
   select * from adm97
      union all 
   select * from adm98
;
quit;

title 'ALLADM2';
proc print data=alladm2;
run;


* EXTRA ... other UNION option ... PATIENTS admitted in 1998, NOT admitted in 1997;

proc sort data=adm97 (keep=pt_id) out=sadm97 nodupkey;
by pt_id;
run;

proc sort data=adm98 (keep=pt_id) out=sadm98 nodupkey;
by pt_id;
run;

data newin98;
merge sadm97 (in=i97) sadm98 (in=i98);
by pt_id;
if ^i97 and i98;
run;

proc print data=newin98;
run;



proc sql;
create table newin98 as
   select pt_id from adm98
      except
   select pt_id from adm97
;
quit;

title 'NEWIN98';
proc print data=newin98;
run;










* STOP;

* EXAMPLE 6A
combine observations  from two tables based on patient id number
;

*
merge assumes data sets are sorted by pt_id and id
default output is all matched + unmatched observations from both data sets
restrict to only matched observations
;

proc sort data=admits out=sadmits;
by pt_id;
run;


proc sort data=patients out=spatients;
by id;
run;


data admits1 ;
merge sadmits   (in=adm
                   keep=pt_id admdate disdate hosp md)
      spatients (in=pts
                   keep=id lastname sex primmd
                   rename=(id=pt_id));
by pt_id ;
* suppress unmatched observations;
if adm;
run;

title 'ADMITS1';
proc print data=admits1;
run;

*
sql inner join makes no assumptions as to sorted data sets
default output is matched observations, no unmatched
;

proc sql ;
create table admits2 as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from admits as a, patients as b
where a.pt_id = b.id
order by a.pt_id, admdate;
quit;

title 'ADMITS2';
proc print data=admits2;
run;

*
alternate sql code - specify inner join - replace WHERE with ON
;

proc sql ;
create table admits2i as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from admits inner join patients
on pt_id = id
order by pt_id, admdate ;
quit;

title 'ADMITS2I';
proc print data=admits2i;
run;

*
alternate sql code - speficy full join to produce default behavior of merge
;

proc sql ;
create table admits2f as
select pt_id, id, admdate, disdate, hosp, md, lastname, sex, primmd
from admits full join patients
on pt_id = id
order by pt_id, admdate ;
quit;

title 'ADMITS2F';
proc print data=admits2f;
run;

*
alternate sql code - speficy right join to produce matches + umatched patients
;

proc sql ;
create table admits2f as
select pt_id, admdate, disdate, hosp, md, lastname, sex, primmd
from admits right join patients
on pt_id = id
order by pt_id, admdate ;
quit;

title 'ADMITS2F';
proc print data=admits2f;
run;

* EXAMPLE 6B
combine observations from three data sets
select only deaths
keep selected variables from all three data set
;

* find patients who died;

data died1 (rename = (disdate=dthdate)) ;
merge admits (in=dth
                 keep=pt_id disdate hosp dest
                 where=(dest='9'))
      patients (in=pts
                   keep=id birthdte
                   rename=(id=pt_id));
by pt_id;
if dth and pts ;
agedth = floor((disdate - birthdte)/365.25) ;
drop dest birthdte ;
run;

proc sort data=died1;
by hosp;
run;

title 'DIED1';
proc print data=died1;
run;

* add hospital information;

data died1b ;
merge died1 (in=dth
             rename=(hosp=hosp_id))
      hospital (in=hsp
                   keep=hosp_id nbeds);
by hosp_id ;
if dth and hsp ;
drop hosp_id;
run;

proc sort data=died1b;
by pt_id ;
run;

title 'DIED1B';
proc print data=died1b;
run;

proc sql ;
create table died2 as
select pt_id, nbeds, disdate as dthdate,
       int((disdate-birthdte)/365.25) as agedth
from admits, hospital, patients
where (pt_id = id) and (hosp = hosp_id) and dest eq '9'
order by pt_id ;
quit;

title 'DIED2';
proc print data=died2;
run;


* EXAMPLE 6C
create a data set with an observation for each hospital
show if a hospital had any admissions (variable hasadmit: 0/no, 1/yes)
;

proc sort data = admits (keep = hosp)
          out  = admits (rename=(hosp=hosp_id))
          nodupkey;
by hosp;
run;

data hosps1 ;
merge hospital (in=hosp)
      admits (in=adm);
by hosp_id ;
if hosp;
* clever - use 0/1 variable and assign value to a new variable;
hasadmit = adm ;
run;

title 'HOSPS1';
proc print data=hosps1;
run;

proc sql ;
create table hosps2 as
select distinct a.*,
       hosp is not missing as hasadmit
from hospital a left join admits b
on a.hosp_id = b.hosp ;
quit;

title 'HOSPS2';
proc print data=hosps2;
run;


* EXAMPLE 6D
create data set with observations for patients treated their own MD
add MD name to the final data set
;

* first find admits treated by primary MD (primary MD info in PATIENTS);

data prim1 (drop = primmd);
merge admits (in=adm
                 keep=pt_id admdate disdate hosp md)
      patients (in=pts
                   keep=id lastname primmd
                   rename=(id=pt_id));
by pt_id ;
if adm and pts and (md eq primmd) ;
run;

proc sort data=prim1;
by md;
run;

* MD names;

data doctors ;
set doctors (keep=md_id lastname);
by md_id ;
if first.md_id ;
run;

title 'DOCTORS';
proc print data=doctors;
run;

* alternative;

proc sort data=doctors (keep=md_id lastname) out=doctors nodupkey;
by md_id;
run;

title 'DOCTORS';
proc print data=doctors;
run;

data prim1a ;
merge prim1 (in=p
             rename=(lastname=ptname md=md_id))
      doctors (rename=(lastname=mdname));
by md_id ;
if p ;
run;

title 'PRIM1A';
proc print data=prim1a;
run;

proc sql ;
create table prim2 as
select pt_id, admdate, disdate, hosp, md_id,
       b.lastname as ptname,
       c.lastname as mdname
from admits a,
     patients b,
     (select distinct md_id, lastname from doctors) c
     where (a.pt_id eq b.id)  and
           (a.md eq b.primmd) and
           (a.md eq c.md_id)
order by a.pt_id, admdate ;
quit;

title 'PRIM2';
proc print data=prim2;
run;


* EXAMPLE 7
create a data set with MDs who had admissions to the VA hospital
;

proc sort data=admits (keep=md hosp)
          out=admits;
by md;
run;

proc sort data=doctors
          out=doctors
          nodupkey ;
by md_id ;
run;

data vadocs1 (drop = hosp);
merge doctors (in=docs
               keep=md_id lastname)
      admits  (in=adm
               where=(hosp = 3)
               rename = (md=md_id)) ;
by md_id;
put first.md_id= last.md_id= docs= adm=;
if docs and adm and first.md_id ;
run;

proc sort data=vadocs1;
by lastname;
run;

title 'VADOCS1';
proc print data=vadocs1;
run;

*
alternative - new data set admits one observation per md/hospital combination
only observations from VA (hosp=3)
;

proc sort data=admits (keep=md hosp where=(hosp=3))
          out=admits (drop=hosp rename=(md=md_id)) nodupkey;
by md hosp;
run;

proc sort data=doctors
          out=doctors (keep=md_id lastname) nodupkey ;
by md_id ;
run;

data vadocs1;
merge doctors (in=docs)
      admits  (in=adm);
by md_id;
if docs and adm;
run;

proc sort data=vadocs1;
by lastname;
run;

title 'VADOCS1';
proc print data=vadocs1;
run;

proc sql;
create table vadocs2 as
select distinct md_id, lastname
from doctors as d
where 3 in (select hosp
            from admits as a
            where d.md_id = a.md)
order by lastname;
quit;

title 'VADOCS2';
proc print data=vadocs2;
run;
